#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))
# 注意：该函数返回的对象不会更新，不建议在循环内调用该方法

# 最近 1 天持仓排名信息，以成交量排序
df = api.query_symbol_ranking("SHFE.ag2504", ranking_type="VOLUME")
print(df.to_string())

# 最近 3 天持仓排名信息，以多头持仓量排序
df = api.query_symbol_ranking("SHFE.ag2504", ranking_type="LONG",days=3)
print(df.to_string())

# 关闭api,释放相应资源
api.close()


#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))

ls = api.query_options("SSE.510050", option_class="CALL", expired=False)  # 所有未下市上交所上证50etf期权
df = api.query_symbol_info(ls)
print(df.to_string())

# 关闭api,释放相应资源
api.close()


#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))
quote = api.get_quote("SHFE.au2504")

# 预计输出的为以au2504现在最新价来比对的认购的平值期权，当没有符合的平值期权时返回为空
# ['SHFE.au2504C680']
ls = api.query_atm_options("SHFE.au2504", quote.last_price, 0, "CALL")
print(ls)

# 预计输出的为以au2504开盘价来比对的认购的实值1档，平值期权，虚值1档，如果没有符合要求的期权则对应栏返回为None，如果有则返回格式例如
# ['SHFE.au2504C680', 'SHFE.au2504C688', 'SHFE.au2504C696']
ls = api.query_atm_options("SHFE.au2504", quote.open, [1,0,-1], "CALL")
print(ls)

# 预计输出沪深300股指期权,2025年12月的虚值1档期权
ls = api.query_atm_options("SSE.000300", quote.last_price, -1, "CALL", exercise_year=2025, exercise_month=12)
print(ls)

# 关闭api,释放相应资源
api.close()



#!usr/bin/env python3
# -*- coding:utf-8 -*-

__author__ = 'mayanqiong'

from tqsdk.tradeable.otg.tqaccount import TqAccount
from tqsdk.tradeable.otg.tqzq import TqZq
from tqsdk.tradeable.otg.tqkq import TqKq, TqKqStock
from tqsdk.tradeable.otg.tqctp import TqCtp
from tqsdk.tradeable.otg.tqrohon import TqRohon
from tqsdk.tradeable.otg.tqjees import TqJees
from tqsdk.tradeable.otg.tqyida import TqYida
from tqsdk.tradeable.otg.tqtradingunit import TqTradingUnit



#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chenli'

import asyncio
import json
import sys
from pathlib import Path
import subprocess

from tqsdk.exceptions import TqContextManagerError


class ZqContext(object):
    def __init__(self, api):
        self._api = api
        self._zq_config_path = Path.home() / ".tqsdk" / "zq" / "config" / "zq.json"

    async def __aenter__(self):
        return self

    async def get_url(self, url_info):
        """无法启动时抛出 TqContextManagerError 例外"""
        try:
            with open(self._zq_config_path, 'r') as file:
                config = json.load(file)
                td_url = config["td_url"]
                interpreter = config["interpreter"]
                if sys.platform.startswith("win"):
                    zq_proc = subprocess.Popen([interpreter, "-m", "tqsdk_zq.cli", "start"], stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
                    returncode = await self._api._loop.run_in_executor(None, lambda: zq_proc.wait())
                else:
                    zq_proc = await asyncio.create_subprocess_exec(interpreter, "-m", "tqsdk_zq.cli", "start", stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
                    returncode = await zq_proc.wait()
                if returncode != 0:
                    raise TqContextManagerError(f"无法拉起 zq 进程, 返回码: {returncode}")
                return td_url
        except (OSError, KeyError, json.JSONDecodeError):
            raise Exception(f"加载配置文件失败: {self._zq_config_path}, 请尝试重新初始化: tqsdk-zq init") from None
        except:
            raise TqContextManagerError("获取交易服务地址失败")
    
    async def __aexit__(self, exc_type, exc, tb):
        pass



from PyInstaller.utils.hooks import collect_data_files, collect_submodules
datas = collect_data_files('tqsdk', includes=['web', 'expired_quotes.json.lzma'])
hiddenimports = collect_submodules('tqsdk_ctpse')



#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'mayanqiong'

import math
from shinny_structlog import ShinnyLoggerAdapter

from tqsdk.baseModule import TqModule
from tqsdk.channel import TqChan
from tqsdk.connect import TqConnect, TsReconnectHandler
from tqsdk.entity import Entity
from tqsdk.objs import Quote
from tqsdk.utils import _query_for_quote
from tqsdk.diff import _get_obj, _merge_diff

class TqTradingStatus(TqModule):
    """
    交易状态模块，建立 websocket 连接
    """

    async def _run(self, api, api_send_chan, api_recv_chan, md_send_chan, md_recv_chan):
        self._api = api
        self._logger = self._api._logger.getChild("TradingStatus")
        self._init_ts_ws = False
        self._api_send_chan = api_send_chan
        self._api_recv_chan = api_recv_chan
        self._md_send_chan = md_send_chan
        self._md_recv_chan = md_recv_chan
        self._ts_send_chan = TqChan(self._api, chan_name="send to ts_reconn")
        self._ts_recv_chan = TqChan(self._api, chan_name="recv from ts_reconn")
        self._data = Entity()
        self._data._instance_entity([])
        self._quote_chan = TqChan(self._api, last_only=True)
        self._prototype = {
            "quotes": {
                "#": Quote(self._api),  # 行情的数据原型
            }
        }
        self._quotes_ready = {}
        self._quotes_unready = {}
        self._tasks = [self._api.create_task(self._symbol_info_watcher())]
        try:
            await super(TqTradingStatus, self)._run(api, api_send_chan, api_recv_chan, md_send_chan, md_recv_chan, self._ts_send_chan, self._ts_recv_chan)
        finally:
            await self._api._cancel_tasks(*self._tasks)

    async def _subscribe_trading_status(self):
        """订阅交易状态服务"""
        ins_list = set()
        for quote in self._quotes_ready.values():
            if quote.ins_class == "OPTION" and quote.underlying_symbol:
                ins_list.add(quote.underlying_symbol)
            else:
                ins_list.add(quote.instrument_id)

        ins_list_str = ",".join(ins_list)
        if ins_list_str:
            await self._ts_send_chan.send({"aid": "subscribe_trading_status", "ins_list": ins_list_str})

    def _extend_option_trading_status(self, diffs):
        """为期权扩展交易状态信息"""
        received_statuses = {
            k: v for d in diffs for k, v in d.get('trading_status', {}).items()
        }
        if not received_statuses:
            return

        extended_diffs = [
            {'trading_status': {
                quote.instrument_id: {
                    'symbol': quote.instrument_id,
                    'trade_status': received_statuses[quote.underlying_symbol]['trade_status']
                }
            }}
            for quote in self._quotes_ready.values()
            if quote.ins_class == 'OPTION' and quote.underlying_symbol in received_statuses
        ]
        diffs.extend(extended_diffs)

    def _normalize_trade_status(self, diffs):
        """标准化交易状态，将非交易状态统一为NOTRADING"""
        for d in diffs:
            for _, ts in d.get('trading_status', {}).items():
                if ts['trade_status'] not in ["AUCTIONORDERING", "CONTINOUS"]:
                    ts['trade_status'] = "NOTRADING"

    async def _query_symbol_info(self, symbols):
        """查询缺少合约信息的quotes"""
        for symbol in symbols:
            self._quotes_unready[symbol]["_listener"].add(self._quote_chan)
        for query_pack in _query_for_quote(list(symbols), self._api._pre20_ins_info.keys()):
            await self._md_send_chan.send(query_pack)

    async def _symbol_info_watcher(self):
        async for _ in self._quote_chan:
            for symbol in await self._unready_to_ready():
                self._quotes_ready[symbol]["_listener"].discard(self._quote_chan)

    async def _unready_to_ready(self):
        ready_delta = {symbol for symbol, quote in self._quotes_unready.items() if not math.isnan(quote.price_tick)}
        for symbol in ready_delta:
            quote = self._quotes_unready.pop(symbol)
            self._quotes_ready[symbol] = quote
        if ready_delta:
            await self._subscribe_trading_status()
        return ready_delta

    async def _handle_recv_data(self, pack, chan):
        """
        处理所有上游收到的数据包
        """
        if pack['aid'] == 'rtn_data':
            if chan == self._md_recv_chan:  # 从行情收到的数据包
                datas = pack.get("data", [])
                self._diffs.extend(datas)
                for d in datas:
                    quotes_diff = d.get("quotes", {})
                    _merge_diff(self._data, {"quotes": {k: quotes_diff[k] for k in quotes_diff.keys() & self._quotes_unready.keys()}}, self._prototype, persist=False, reduce_diff=False)
            elif chan == self._ts_recv_chan:  # 从交易状态服务收到的数据包
                diffs = pack.get('data', [])
                self._extend_option_trading_status(diffs)
                self._normalize_trade_status(diffs)
                self._diffs.extend(diffs)
        else:
            await self._api_recv_chan.send(pack)

    async def _handle_req_data(self, pack):
        """处理所有下游发送的非 peek_message 数据包"""
        if pack['aid'] == 'subscribe_trading_status':
            if self._init_ts_ws is False:
                self._init_ts_ws = True
                self._create_ts_run()
            unseen = set(pack["ins_list"].split(",")) - self._quotes_ready.keys() - self._quotes_unready.keys()
            for symbol in unseen:
                self._quotes_unready[symbol] = _get_obj(self._data, ["quotes", symbol], self._prototype["quotes"]["#"])
            await self._query_symbol_info(unseen - await self._unready_to_ready())
        else:
            await self._md_send_chan.send(pack)

    def _create_ts_run(self):
        ts_url = "wss://trading-status.shinnytech.com/status"
        conn_logger = self._api._logger.getChild("TqConnect")
        ws_ts_send_chan = TqChan(self._api, chan_name="send to ts")
        ws_ts_recv_chan = TqChan(self._api, chan_name="recv from ts")
        ws_ts_send_chan._logger_bind(chan_from="ts_reconn", url=ts_url)
        ws_ts_recv_chan._logger_bind(chan_to="ts_reconn", url=ts_url)
        conn = TqConnect(logger=ShinnyLoggerAdapter(conn_logger, url=ts_url), conn_id="ts")
        self._tasks.append(self._api.create_task(conn._run(self._api, ts_url, ws_ts_send_chan, ws_ts_recv_chan), _caller_api=True))
        ts_reconnect = TsReconnectHandler(logger=ShinnyLoggerAdapter(self._logger.getChild("TsReconnect"), url=ts_url))
        self._ts_send_chan._logger_bind(chan_from="ts", url=ts_url)
        self._ts_recv_chan._logger_bind(chan_to="ts", url=ts_url)
        self._tasks.append(self._api.create_task(ts_reconnect._run(self._api, self._ts_send_chan, self._ts_recv_chan, ws_ts_send_chan, ws_ts_recv_chan), _caller_api=True))


#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))

quote = api.get_quote("SHFE.au2504")
in_money_options, at_money_options, out_of_money_options = api.query_all_level_options("SHFE.au2504", quote.last_price, "CALL")
# 实值期权列表
print(in_money_options)
# 平值期权列表
print(at_money_options)
# 虚值期权列表
print(out_of_money_options)

# 关闭api,释放相应资源
api.close()



#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))

# 不推荐使用以下方式获取符合某种条件的合约列表，推荐使用接口来完成此功能。
# ls = [k for k,v in api._data["quotes"].items() if k.startswith("KQ.m")]
# print(ls)

# au 品种的全部合约，包括已下市以及未下市合约
ls = api.query_quotes(ins_class="FUTURE", product_id="au")
print(ls)

# au、cu 品种的全部未下市合约合约
ls = api.query_quotes(ins_class=["FUTURE"], product_id=["au", "cu"], expired=False)
print(ls)

# au 品种指数合约
ls = api.query_quotes(ins_class="INDEX", product_id="au")
print(ls)

# 全部主连合约
ls = api.query_quotes(ins_class="CONT")
print(ls)

# au 品种主连合约
ls = api.query_quotes(ins_class="CONT", product_id="au")
print(ls)

# 上期所带夜盘的期货合约列表
ls = api.query_quotes(ins_class="FUTURE", exchange_id="SHFE", has_night=True)
print(ls)

# au 品种的全部未下市合约、指数、主连
ls = api.query_quotes(product_id="au", expired=False)
print(ls)

# 上海交易所股票代码列表
ls = api.query_quotes(ins_class="STOCK", exchange_id="SSE", expired=False)
print(ls)

# 上海交易所基金代码列表
ls = api.query_quotes(ins_class="FUND", exchange_id="SSE", expired=False)
print(ls)

# 关闭api,释放相应资源
api.close()


#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from datetime import datetime
from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))
# 注意：该函数返回的对象不会更新，不建议在循环内调用该方法

# 最近3天结算价信息
df = api.query_symbol_settlement("SHFE.ag2504", days=3)
print(df.to_string())

# 查询从2025年2月10日开始两天的结算价信息
df = api.query_symbol_settlement("SHFE.ag2504", days=2, start_dt=datetime(2025,2,10).date())
print(df.to_string())

# 关闭api,释放相应资源
api.close()


#!/usr/bin/env python
#  -*- coding: utf-8 -*-
__author__ = 'chaos'

from tqsdk import TqApi, TqAuth

api = TqApi(auth=TqAuth("快期账户", "账户密码"))

# 标的为 "SHFE.au2504" 的所有期权
ls = api.query_options("SHFE.au2504")
print(ls)

# 标的为 "SHFE.au2504" 的看跌期权
ls = api.query_options("SHFE.au2504", option_class="PUT")
print(ls)

# 标的为 "SHFE.au2504" 的看跌期权, 未下市的
ls = api.query_options("SHFE.au2504", option_class="PUT", expired=False)
print(ls)

# 标的为 "SHFE.au2504" 、行权价为 340 的期权
ls = api.query_options("SHFE.au2504", strike_price=340)
print(ls)

# 中金所沪深300股指期权
ls = api.query_options("SSE.000300")
print(ls)

# 上交所沪深300etf期权
ls = api.query_options("SSE.510300")
print(ls)

# 上交所沪深300etf期权, 限制条件 2020 年 12 月份行权
ls = api.query_options("SSE.510300", exercise_year=2020, exercise_month=12)
print(ls)

# 关闭api,释放相应资源
api.close()


#!usr/bin/env python3
# -*- coding:utf-8 -*-
__author__ = 'yanqiong'


import sgqlc.types
from sgqlc.operation import Fragment


########################################################################
# Monkey patching 检查请求的参数是否合法，与 api.query_graphql 函数的校验规则保持一致
########################################################################
_origin__to_graphql_input__ = sgqlc.types.Arg.__to_graphql_input__


def _tqsdk__to_graphql_input__(self, value, *args, **kwargs):
    if value == "" or isinstance(value, list) and (any([s == "" for s in value]) or len(value) == 0):
        raise Exception(f"variables 中变量值不支持空字符串、空列表或者列表中包括空字符串。")
    return _origin__to_graphql_input__(self, value, *args, **kwargs)


sgqlc.types.Arg.__to_graphql_input__ = _tqsdk__to_graphql_input__


ins_schema = sgqlc.types.Schema()


########################################################################
# Scalars and Enumerations
########################################################################
Boolean = sgqlc.types.Boolean


class Category(sgqlc.types.Enum):
    '''全部板块

    Enumeration Choices:

    * `AGRICULTURAL`: 农副
    * `CHEMICAL`: 化工
    * `COAL`: 煤炭
    * `EQUITY_INDEX`: 股指期货
    * `FERROUS`: 黑色金属
    * `GRAIN`: 谷物
    * `GREASE`: 油脂油料
    * `LIGHT_INDUSTRY`: 轻工
    * `NONFERROUS_METALS`: 有色金属
    * `OIL`: 石油
    * `PRECIOUS_METALS`: 贵金属
    * `SOFT_COMMODITY`: 软商
    * `TREASURY_BOND`: 国债期货
    '''
    __schema__ = ins_schema
    __choices__ = ('AGRICULTURAL', 'CHEMICAL', 'COAL', 'EQUITY_INDEX', 'FERROUS', 'GRAIN', 'GREASE', 'LIGHT_INDUSTRY', 'NONFERROUS_METALS', 'OIL', 'PRECIOUS_METALS', 'SOFT_COMMODITY', 'TREASURY_BOND')


class Class(sgqlc.types.Enum):
    '''全部类别

    Enumeration Choices:

    * `BOND`: 债券
    * `COMBINE`: 组合
    * `CONT`: 连续
    * `FUND`: 基金
    * `FUTURE`: 期货
    * `INDEX`: 指数
    * `OPTION`: 期权
    * `SPOT`: 现货
    * `STOCK`: 股票
    '''
    __schema__ = ins_schema
    __choices__ = ('BOND', 'COMBINE', 'CONT', 'FUND', 'FUTURE', 'INDEX', 'OPTION', 'SPOT', 'STOCK')


Float = sgqlc.types.Float

Int = sgqlc.types.Int


class Int64(sgqlc.types.Scalar):
    __schema__ = ins_schema


String = sgqlc.types.String


########################################################################
# Input Objects
########################################################################

########################################################################
# Output Objects and Interfaces
########################################################################
class basic(sgqlc.types.Interface):
    '''基础信息'''
    __schema__ = ins_schema
    __field_names__ = ('instrument_name_wh', 'price_decs', 'price_tick', 'ins_id', 'instrument_id', 'derivatives', 'exchange_id',
                       'instrument_name', 'trading_time', 'class_', 'py_wh', 'english_name', 'derivative', 'trading_day')
    instrument_name_wh = sgqlc.types.Field(String, graphql_name='instrument_name_wh')
    '''合约名称（文华风格）'''

    price_decs = sgqlc.types.Field(Int, graphql_name='price_decs')
    '''价格小数位数'''

    price_tick = sgqlc.types.Field(Float, graphql_name='price_tick')
    '''最小变动价位'''

    ins_id = sgqlc.types.Field(String, graphql_name='ins_id')
    '''合约代码（不含交易所）'''

    instrument_id = sgqlc.types.Field(String, graphql_name='instrument_id')
    '''合约代码'''

    derivatives = sgqlc.types.Field('derivativeConnection', graphql_name='derivatives', args=sgqlc.types.ArgDict((
        ('class_', sgqlc.types.Arg(sgqlc.types.list_of(Class), graphql_name='class', default=None)),
        ('exchange_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exchange_id', default=None)),
        ('expired', sgqlc.types.Arg(Boolean, graphql_name='expired', default=None)),
        ('timestamp', sgqlc.types.Arg(Int64, graphql_name='timestamp', default=None)),
    ))
    )
    '''衍生品

    Arguments:

    * `class_` (`[Class]`): placeholder
    * `exchange_id` (`[String]`): placeholder
    * `expired` (`Boolean`)
    * `timestamp` (`Int64`)
    '''

    exchange_id = sgqlc.types.Field(String, graphql_name='exchange_id')
    '''交易所代码'''

    instrument_name = sgqlc.types.Field(String, graphql_name='instrument_name')
    '''合约名称'''

    trading_time = sgqlc.types.Field('tradingTime', graphql_name='trading_time')
    '''交易时间'''

    class_ = sgqlc.types.Field(String, graphql_name='class')
    '''类别'''

    py_wh = sgqlc.types.Field(String, graphql_name='py_wh')
    '''拼音（文华风格）'''

    english_name = sgqlc.types.Field(String, graphql_name='english_name')
    '''英文名称'''

    derivative = sgqlc.types.Field('derivativeConnection', graphql_name='derivative', args=sgqlc.types.ArgDict((
        ('class_', sgqlc.types.Arg(Class, graphql_name='class', default=None)),
        ('exchange_id', sgqlc.types.Arg(String, graphql_name='exchange_id', default=None)),
        ('expired', sgqlc.types.Arg(Boolean, graphql_name='expired', default=None)),
    ))
    )
    '''衍生品（旧）

    Arguments:

    * `class_` (`Class`): placeholder
    * `exchange_id` (`String`): placeholder
    * `expired` (`Boolean`)
    '''

    trading_day = sgqlc.types.Field(String, graphql_name='trading_day')
    '''交易日'''


class derivative(sgqlc.types.Interface):
    '''衍生品信息'''
    __schema__ = ins_schema
    __field_names__ = ('underlying',)
    underlying = sgqlc.types.Field('derivativeConnection', graphql_name='underlying')
    '''标的合约'''


class securities(sgqlc.types.Interface):
    '''证券信息'''
    __schema__ = ins_schema
    __field_names__ = ('first_trading_datetime', 'buy_volume_unit', 'sell_volume_unit', 'status', 'public_float_share_quantity', 'currency', 'face_value', 'first_trading_day')
    first_trading_datetime = sgqlc.types.Field(Int64, graphql_name='first_trading_datetime')
    '''起始交易时间'''

    buy_volume_unit = sgqlc.types.Field(Float, graphql_name='buy_volume_unit')
    '''买入单位'''

    sell_volume_unit = sgqlc.types.Field(Float, graphql_name='sell_volume_unit')
    '''卖出单位'''

    status = sgqlc.types.Field(String, graphql_name='status')
    '''状态'''

    public_float_share_quantity = sgqlc.types.Field(Int64, graphql_name='public_float_share_quantity')
    '''流通股本'''

    currency = sgqlc.types.Field(String, graphql_name='currency')
    '''币种'''

    face_value = sgqlc.types.Field(Float, graphql_name='face_value')
    '''面值'''

    first_trading_day = sgqlc.types.Field(Int64, graphql_name='first_trading_day')
    '''首个交易日'''


class tradeable(sgqlc.types.Interface):
    '''交易信息'''
    __schema__ = ins_schema
    __field_names__ = ('upper_limit', 'lower_limit', 'volume_multiple', 'quote_multiple', 'pre_close')
    upper_limit = sgqlc.types.Field(Float, graphql_name='upper_limit')
    '''涨停板价'''

    lower_limit = sgqlc.types.Field(Float, graphql_name='lower_limit')
    '''跌停板价'''

    volume_multiple = sgqlc.types.Field(Float, graphql_name='volume_multiple')
    '''合约乘数（交易单位 / 报价单位）'''

    quote_multiple = sgqlc.types.Field(Float, graphql_name='quote_multiple')
    '''报价乘数（成交量单位 / 报价单位）'''

    pre_close = sgqlc.types.Field(Float, graphql_name='pre_close')
    '''昨收盘价'''


class categoryInfo(sgqlc.types.Type):
    '''板块信息'''
    __schema__ = ins_schema
    __field_names__ = ('id', 'name')
    id = sgqlc.types.Field(String, graphql_name='id')
    '''板块ID'''

    name = sgqlc.types.Field(String, graphql_name='name')
    '''板块中文名称'''


class derivativeConnection(sgqlc.types.Type):
    '''衍生品关系'''
    __schema__ = ins_schema
    __field_names__ = ('count', 'edges')
    count = sgqlc.types.Field(Int, graphql_name='count')

    edges = sgqlc.types.Field(sgqlc.types.list_of('derivativeEdges'), graphql_name='edges')


class derivativeEdges(sgqlc.types.Type):
    '''衍生品'''
    __schema__ = ins_schema
    __field_names__ = ('node', 'underlying_multiple')
    node = sgqlc.types.Field('allClassUnion', graphql_name='node')
    '''衍生品'''

    underlying_multiple = sgqlc.types.Field(Float, graphql_name='underlying_multiple')


class rootQuery(sgqlc.types.Type):
    '''symbol_info 为旧版单查询接口，multi_symbol_info 为新版多查询接口，即前者的兼容升级版本'''
    __schema__ = ins_schema
    __field_names__ = ('multi_symbol_info', 'symbol_info')
    multi_symbol_info = sgqlc.types.Field(sgqlc.types.list_of('allClassUnion'), graphql_name='multi_symbol_info', args=sgqlc.types.ArgDict((
        ('instrument_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='instrument_id', default=None)),
        ('class_', sgqlc.types.Arg(sgqlc.types.list_of(Class), graphql_name='class', default=None)),
        ('exchange_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='exchange_id', default=None)),
        ('product_id', sgqlc.types.Arg(sgqlc.types.list_of(String), graphql_name='product_id', default=None)),
        ('expired', sgqlc.types.Arg(Boolean, graphql_name='expired', default=None)),
        ('has_night', sgqlc.types.Arg(Boolean, graphql_name='has_night', default=None)),
        ('has_derivatives', sgqlc.types.Arg(Boolean, graphql_name='has_derivatives', default=None)),
        ('categories', sgqlc.types.Arg(sgqlc.types.list_of(Category), graphql_name='categories', default=None)),
        ('timestamp', sgqlc.types.Arg(Int64, graphql_name='timestamp', default=None)),
    ))
    )
    '''Arguments:

    * `instrument_id` (`[String]`): 合约代码
    * `class_` (`[Class]`): 类别
    * `exchange_id` (`[String]`): 交易所代码
    * `product_id` (`[String]`): 品种代码
    * `expired` (`Boolean`): 是否已到期
    * `has_night` (`Boolean`): 是否有夜盘
    * `has_derivatives` (`Boolean`): 是否有衍生品
    * `categories` (`[Category]`): 所属板块
    * `timestamp` (`Int64`): 回测时间点
    '''

    symbol_info = sgqlc.types.Field(sgqlc.types.list_of('allClassUnion'), graphql_name='symbol_info', args=sgqlc.types.ArgDict((
        ('instrument_id', sgqlc.types.Arg(String, graphql_name='instrument_id', default=None)),
        ('class_', sgqlc.types.Arg(Class, graphql_name='class', default=None)),
        ('exchange_id', sgqlc.types.Arg(String, graphql_name='exchange_id', default=None)),
        ('product_id', sgqlc.types.Arg(String, graphql_name='product_id', default=None)),
        ('expired', sgqlc.types.Arg(Boolean, graphql_name='expired', default=None)),
        ('has_night', sgqlc.types.Arg(Boolean, graphql_name='has_night', default=None)),
        ('has_derivatives', sgqlc.types.Arg(Boolean, graphql_name='has_derivatives', default=None)),
    ))
    )
    '''Arguments:

    * `instrument_id` (`String`)
    * `class_` (`Class`)
    * `exchange_id` (`String`)
    * `product_id` (`String`)
    * `expired` (`Boolean`)
    * `has_night` (`Boolean`)
    * `has_derivatives` (`Boolean`)
    '''


class tradingTime(sgqlc.types.Type):
    '''交易时间'''
    __schema__ = ins_schema
    __field_names__ = ('day', 'night')
    day = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.list_of(String)), graphql_name='day')
    '''白盘交易时间'''

    night = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.list_of(String)), graphql_name='night')
    '''夜盘交易时间'''


class bond(sgqlc.types.Type, basic, tradeable, securities):
    '''债券'''
    __schema__ = ins_schema
    __field_names__ = ('maturity_date', 'maturity_datetime')
    maturity_date = sgqlc.types.Field(Int64, graphql_name='maturity_date')
    '''到期日'''

    maturity_datetime = sgqlc.types.Field(Int64, graphql_name='maturity_datetime')
    '''到期时间'''


class combine(sgqlc.types.Type, basic):
    '''组合'''
    __schema__ = ins_schema
    __field_names__ = ('close_max_limit_order_volume', 'close_max_market_order_volume', 'close_min_limit_order_volume', 'close_min_market_order_volume', 'expire_datetime', 'expired', 'leg1', 'leg2', 'max_limit_order_volume',
                       'max_market_order_volume', 'min_limit_order_volume', 'min_market_order_volume', 'open_max_limit_order_volume', 'open_max_market_order_volume', 'open_min_limit_order_volume', 'open_min_market_order_volume', 'product_id')
    close_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_limit_order_volume')
    '''平仓限价单最大下单量'''

    close_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_market_order_volume')
    '''平仓市价单最大下单量'''

    close_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_limit_order_volume')
    '''平仓限价单最小下单量'''

    close_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_market_order_volume')
    '''平仓市价单最小下单量'''

    expire_datetime = sgqlc.types.Field(Int64, graphql_name='expire_datetime')
    '''到期时间'''

    expired = sgqlc.types.Field(Boolean, graphql_name='expired')
    '''是否到期'''

    leg1 = sgqlc.types.Field('allClassUnion', graphql_name='leg1')
    '''组合单腿1'''

    leg2 = sgqlc.types.Field('allClassUnion', graphql_name='leg2')
    '''组合单腿2'''

    max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='max_limit_order_volume')
    '''限价单最大下单量'''

    max_market_order_volume = sgqlc.types.Field(Int, graphql_name='max_market_order_volume')
    '''市价单最大下单量'''

    min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='min_limit_order_volume')
    '''限价单最小下单量'''

    min_market_order_volume = sgqlc.types.Field(Int, graphql_name='min_market_order_volume')
    '''市价单最小下单量'''

    open_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_limit_order_volume')
    '''开仓限价单最大下单量'''

    open_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_market_order_volume')
    '''开仓市价单最大下单量'''

    open_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_limit_order_volume')
    '''开仓限价单最小下单量'''

    open_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_market_order_volume')
    '''开仓市价单最小下单量'''

    product_id = sgqlc.types.Field(String, graphql_name='product_id')
    '''品种代码'''


class cont(sgqlc.types.Type, basic, tradeable, derivative):
    '''连续'''
    __schema__ = ins_schema
    __field_names__ = ()


class fund(sgqlc.types.Type, basic, tradeable, securities):
    '''基金'''
    __schema__ = ins_schema
    __field_names__ = ('cash_dividend_ratio',)
    cash_dividend_ratio = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='cash_dividend_ratio')
    '''现金分红'''


class future(sgqlc.types.Type, basic, tradeable):
    '''期货'''
    __schema__ = ins_schema
    __field_names__ = ('categories', 'close_max_limit_order_volume', 'close_max_market_order_volume', 'close_min_limit_order_volume', 'close_min_market_order_volume', 'commission', 'delivery_month', 'delivery_year', 'expire_datetime', 'expired', 'margin', 'max_limit_order_volume', 'max_market_order_volume', 'min_limit_order_volume',
                       'min_market_order_volume', 'mmsa', 'open_max_limit_order_volume', 'open_max_market_order_volume', 'open_min_limit_order_volume', 'open_min_market_order_volume', 'position_limit', 'pre_open_interest', 'pre_open_interest2', 'product_id', 'product_short_name', 'product_short_name_wh', 'settlement_price')
    categories = sgqlc.types.Field(sgqlc.types.list_of(categoryInfo), graphql_name='categories')
    '''所属板块'''

    close_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_limit_order_volume')
    '''平仓限价单最大下单量'''

    close_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_market_order_volume')
    '''平仓市价单最大下单量'''

    close_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_limit_order_volume')
    '''平仓限价单最小下单量'''

    close_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_market_order_volume')
    '''平仓市价单最小下单量'''

    commission = sgqlc.types.Field(Float, graphql_name='commission')
    '''手续费'''

    delivery_month = sgqlc.types.Field(Int, graphql_name='delivery_month')
    '''交割月份'''

    delivery_year = sgqlc.types.Field(Int, graphql_name='delivery_year')
    '''交割年份'''

    expire_datetime = sgqlc.types.Field(Int64, graphql_name='expire_datetime')
    '''到期时间'''

    expired = sgqlc.types.Field(Boolean, graphql_name='expired')
    '''是否到期'''

    margin = sgqlc.types.Field(Float, graphql_name='margin')
    '''保证金'''

    max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='max_limit_order_volume')
    '''限价单最大下单量'''

    max_market_order_volume = sgqlc.types.Field(Int, graphql_name='max_market_order_volume')
    '''市价单最大下单量'''

    min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='min_limit_order_volume')
    '''限价单最小下单量'''

    min_market_order_volume = sgqlc.types.Field(Int, graphql_name='min_market_order_volume')
    '''市价单最小下单量'''

    mmsa = sgqlc.types.Field(Boolean, graphql_name='mmsa')
    '''单向大边'''

    open_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_limit_order_volume')
    '''开仓限价单最大下单量'''

    open_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_market_order_volume')
    '''开仓市价单最大下单量'''

    open_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_limit_order_volume')
    '''开仓限价单最小下单量'''

    open_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_market_order_volume')
    '''开仓市价单最小下单量'''

    position_limit = sgqlc.types.Field(Int, graphql_name='position_limit')
    '''持仓限额手数，为 null 时表示无该信息，为 0 则不允许持仓'''

    pre_open_interest = sgqlc.types.Field(Int64, graphql_name='pre_open_interest')
    '''昨持仓量（SHFE/INE/CZCE/DCE 为双边计量，其余为单边计量）'''

    pre_open_interest2 = sgqlc.types.Field(Int64, graphql_name='pre_open_interest2')
    '''昨持仓量（单边计量）'''

    product_id = sgqlc.types.Field(String, graphql_name='product_id')
    '''品种代码'''

    product_short_name = sgqlc.types.Field(String, graphql_name='product_short_name')
    '''品种简称'''

    product_short_name_wh = sgqlc.types.Field(String, graphql_name='product_short_name_wh')
    '''品种简称（文华风格）'''

    settlement_price = sgqlc.types.Field(Float, graphql_name='settlement_price')
    '''结算价'''


class index(sgqlc.types.Type, basic):
    '''指数'''
    __schema__ = ins_schema
    __field_names__ = ('index_multiple',)
    index_multiple = sgqlc.types.Field(Float, graphql_name='index_multiple')
    '''指数乘数（指数成交量单位 / 成份股报价单位）'''


class option(sgqlc.types.Type, basic, tradeable, derivative):
    '''期权'''
    __schema__ = ins_schema
    __field_names__ = ('call_or_put', 'close_max_limit_order_volume', 'close_max_market_order_volume', 'close_min_limit_order_volume', 'close_min_market_order_volume', 'exercise_type', 'expire_datetime', 'expired', 'last_exercise_datetime', 'last_exercise_day', 'max_limit_order_volume', 'max_market_order_volume',
                       'min_limit_order_volume', 'min_market_order_volume', 'open_max_limit_order_volume', 'open_max_market_order_volume', 'open_min_limit_order_volume', 'open_min_market_order_volume', 'position_limit', 'pre_open_interest', 'pre_open_interest2', 'product_short_name', 'settlement_price', 'strike_price')
    call_or_put = sgqlc.types.Field(String, graphql_name='call_or_put')
    '''认购/认沽'''

    close_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_limit_order_volume')
    '''平仓限价单最大下单量'''

    close_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_max_market_order_volume')
    '''平仓市价单最大下单量'''

    close_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_limit_order_volume')
    '''平仓限价单最小下单量'''

    close_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='close_min_market_order_volume')
    '''平仓市价单最小下单量'''

    exercise_type = sgqlc.types.Field(String, graphql_name='exercise_type')
    '''行权方式'''

    expire_datetime = sgqlc.types.Field(Int64, graphql_name='expire_datetime')
    '''到期时间'''

    expired = sgqlc.types.Field(Boolean, graphql_name='expired')
    '''是否到期'''

    last_exercise_datetime = sgqlc.types.Field(Int64, graphql_name='last_exercise_datetime')
    '''最后行权时间'''

    last_exercise_day = sgqlc.types.Field(Int64, graphql_name='last_exercise_day')
    '''最后行权日'''

    max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='max_limit_order_volume')
    '''限价单最大下单量'''

    max_market_order_volume = sgqlc.types.Field(Int, graphql_name='max_market_order_volume')
    '''市价单最大下单量'''

    min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='min_limit_order_volume')
    '''限价单最小下单量'''

    min_market_order_volume = sgqlc.types.Field(Int, graphql_name='min_market_order_volume')
    '''市价单最小下单量'''

    open_max_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_limit_order_volume')
    '''开仓限价单最大下单量'''

    open_max_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_max_market_order_volume')
    '''开仓市价单最大下单量'''

    open_min_limit_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_limit_order_volume')
    '''开仓限价单最小下单量'''

    open_min_market_order_volume = sgqlc.types.Field(Int, graphql_name='open_min_market_order_volume')
    '''开仓市价单最小下单量'''

    position_limit = sgqlc.types.Field(Int, graphql_name='position_limit')
    '''持仓限额手数，为 null 时表示无该信息，为 0 则不允许持仓'''

    pre_open_interest = sgqlc.types.Field(Int64, graphql_name='pre_open_interest')
    '''昨持仓量（SHFE/INE/CZCE/DCE 为双边计量，其余为单边计量）'''

    pre_open_interest2 = sgqlc.types.Field(Int64, graphql_name='pre_open_interest2')
    '''昨持仓量（单边计量）'''

    product_short_name = sgqlc.types.Field(String, graphql_name='product_short_name')
    '''品种简称'''

    settlement_price = sgqlc.types.Field(Float, graphql_name='settlement_price')
    '''结算价'''

    strike_price = sgqlc.types.Field(Float, graphql_name='strike_price')
    '''行权价'''


class spot(sgqlc.types.Type, basic, tradeable):
    '''现货'''
    __schema__ = ins_schema
    __field_names__ = ()


class stock(sgqlc.types.Type, basic, tradeable, securities):
    '''股票'''
    __schema__ = ins_schema
    __field_names__ = ('cash_dividend_ratio', 'stock_dividend_ratio')
    cash_dividend_ratio = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='cash_dividend_ratio')
    '''现金分红'''

    stock_dividend_ratio = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='stock_dividend_ratio')
    '''股票分红'''


########################################################################
# Unions
########################################################################
class allClassUnion(sgqlc.types.Union):
    '''分类型结果集'''
    __schema__ = ins_schema
    __types__ = (future, index, option, combine, spot, cont, stock, bond, fund)


########################################################################
# Schema Entry Points
########################################################################
ins_schema.query_type = rootQuery
ins_schema.mutation_type = None
ins_schema.subscription_type = None


########################################################################
# Fragments
########################################################################
basic_frag = Fragment(basic, 'basic')
basic_frag.instrument_id()
basic_frag.exchange_id()
basic_frag.instrument_name()
basic_frag.english_name()
basic_frag.class_()
basic_frag.price_tick()
basic_frag.price_decs()
basic_frag.trading_day()
basic_frag.trading_time().day()
basic_frag.trading_time().night()

stock_frag = Fragment(stock, 'stock')
stock_frag.stock_dividend_ratio()
stock_frag.cash_dividend_ratio()

fund_frag = Fragment(fund, 'fund')
fund_frag.cash_dividend_ratio()

bond_frag = Fragment(bond, 'bond')
bond_frag.maturity_datetime()

tradeable_frag = Fragment(tradeable, 'tradeable')
tradeable_frag.pre_close()
tradeable_frag.volume_multiple()
tradeable_frag.quote_multiple()
tradeable_frag.upper_limit()
tradeable_frag.lower_limit()

index_frag = Fragment(index, 'index')
index_frag.index_multiple()

securities_frag = Fragment(securities, 'securities')
securities_frag.currency()
securities_frag.face_value()
securities_frag.first_trading_datetime()
securities_frag.buy_volume_unit()
securities_frag.sell_volume_unit()
securities_frag.status()
securities_frag.public_float_share_quantity()

future_frag = Fragment(future, 'future')
future_frag.pre_open_interest()
future_frag.expired()
future_frag.product_id()
future_frag.product_short_name()
future_frag.delivery_year()
future_frag.delivery_month()
future_frag.expire_datetime()
future_frag.settlement_price()
future_frag.max_market_order_volume()
future_frag.max_limit_order_volume()
future_frag.min_market_order_volume()
future_frag.min_limit_order_volume()
future_frag.open_max_market_order_volume()
future_frag.open_max_limit_order_volume()
future_frag.open_min_market_order_volume()
future_frag.open_min_limit_order_volume()
future_frag.margin()
future_frag.commission()
future_frag.mmsa()
future_frag.categories()
future_frag.position_limit()

option_frag = Fragment(option, 'option')
option_frag.pre_open_interest()
option_frag.expired()
option_frag.product_short_name()
option_frag.expire_datetime()
option_frag.last_exercise_datetime()
option_frag.settlement_price()
option_frag.max_market_order_volume()
option_frag.max_limit_order_volume()
option_frag.min_market_order_volume()
option_frag.min_limit_order_volume()
option_frag.open_max_market_order_volume()
option_frag.open_max_limit_order_volume()
option_frag.open_min_market_order_volume()
option_frag.open_min_limit_order_volume()
option_frag.strike_price()
option_frag.call_or_put()
option_frag.exercise_type()
option_frag.position_limit()

combine_frag = Fragment(combine, 'combine')
combine_frag.expired()
combine_frag.product_id()
combine_frag.expire_datetime()
combine_frag.max_market_order_volume()
combine_frag.max_limit_order_volume()
combine_frag.min_market_order_volume()
combine_frag.min_limit_order_volume()
combine_frag.open_max_market_order_volume()
combine_frag.open_max_limit_order_volume()
combine_frag.open_min_market_order_volume()
combine_frag.open_min_limit_order_volume()
combine_frag.leg1().__as__(basic).instrument_id()
combine_frag.leg2().__as__(basic).instrument_id()

derivative_frag = Fragment(derivative, 'derivative')
derivative_frag.underlying()
derivative_frag.underlying().count()
derivative_frag.underlying().edges().underlying_multiple()
derivative_frag.underlying().edges().node().__fragment__(basic_frag)
derivative_frag.underlying().edges().node().__fragment__(stock_frag)
derivative_frag.underlying().edges().node().__fragment__(fund_frag)
derivative_frag.underlying().edges().node().__fragment__(bond_frag)
derivative_frag.underlying().edges().node().__fragment__(tradeable_frag)
derivative_frag.underlying().edges().node().__fragment__(index_frag)
derivative_frag.underlying().edges().node().__fragment__(securities_frag)
derivative_frag.underlying().edges().node().__fragment__(future_frag)


def _add_all_frags(field):
    field.__fragment__(basic_frag)
    field.__fragment__(stock_frag)
    field.__fragment__(fund_frag)
    field.__fragment__(bond_frag)
    field.__fragment__(tradeable_frag)
    field.__fragment__(index_frag)
    field.__fragment__(securities_frag)
    field.__fragment__(future_frag)
    field.__fragment__(option_frag)
    field.__fragment__(combine_frag)
    field.__fragment__(derivative_frag)



#!usr/bin/env python3
# -*- coding:utf-8 -*-
__author__ = 'chenli'

import hashlib
from typing import Optional, List
import inspect

from tqsdk.tradeable.otg.base_otg import BaseOtg
from tqsdk.tradeable.mixin import FutureMixin


class TqTradingUnit(BaseOtg, FutureMixin):
    """交易单元类"""

    def __init__(self, account_id: str, tags: Optional[List[str]]=None) -> None:
        """
        创建交易单元实例

        Args:
            account_id (str): 众期子账户

            tags (None/list of str): 策略标签, 默认为调用 TqTradingUnit 的文件名及 account_id

        Example1::

            from tqsdk import TqApi, TqTradingUnit, TqAuth
            account = TqTradingUnit(account_id="众期子账户", tags=["铜品种策略", "套利策略"])
            api = TqApi(account, auth=TqAuth("快期账户", "账户密码"))

        """
        self._tags = self._convert_tags(tags, account_id)
        super(TqTradingUnit, self).__init__(broker_id="", account_id=account_id, password="tqsdk_zq", td_url="zq://localhost/")

    def _convert_tags(self, tags: Optional[List[str]], account_id: str) -> list:
        if tags is None:
            frame = inspect.stack()[1]
            filename = frame.filename
            strategy_set = {filename, account_id}
        else:
            strategy_set = set(tags)
        return list(strategy_set)

    @property
    def _account_auth(self):
        return {
            "feature": "tq_trading_unit",
            "account_id": None,
            "auto_add": False,
        }

    def _get_account_key(self):
        s = self._account_id + self._td_url
        return hashlib.md5(s.encode('utf-8')).hexdigest()

    async def _send_login_pack(self):
        req = {
            "aid": "req_login",
            "bid": "tqsdk_zq",
            "user_name": self._account_id,
            "password": "tqsdk_zq",
            "tags": self._tags
        }
        await self._td_send_chan.send(req)
